@@ -10,7 +10,7 @@ How it works: the library is composed of 2 major parts:
   * Construction phase (there are no bases). Make new instances of the `Factory` with new state.
   * Usage phase. When subclassed (there are bases) it will use the sealer to return the final class.
 """
-import re
+import sys
 from itertools import chain
 from operator import itemgetter
 try:
@@ -18,8 +18,9 @@ try:
 except ImportError:
     from .py2ordereddict import OrderedDict
 
-__version__ = "1.0.3"
+__version__ = "2.0.0"
 
+PY3 = sys.version_info[0] == 3
 MISSING = object()
 
 
@@ -28,33 +29,39 @@ class __base__(object):
         pass
 
 
+def _make_init_func(required, defaults, everything):
+    parts = [
+        'def __init__(self'
+    ]
+    for var in everything:
+        if var in defaults:
+            parts.append(', {0}={0}'.format(var))
+        else:
+            parts.append(', {0}'.format(var))
+    parts.append('):\n')
+    for var in everything:
+        parts.append('    self.{0} = {0}\n'.format(var))
+    parts.append('    super(FieldsBase, self).__init__(')
+    parts.append(', '.join('{0}={0}'.format(var) for var in everything))
+    parts.append(')\n')
+    local_namespace = dict(defaults)
+    global_namespace = dict(super=super)
+    code = ''.join(parts)
+    if PY3:
+        exec(code, global_namespace, local_namespace)
+    else:
+        exec("exec code in global_namespace, local_namespace")
+    return global_namespace, local_namespace
+
+
 def class_sealer(required, defaults, everything):
     """
     This sealer make a normal container class. It's mutable and supports arguments with default values.
     """
-    class FieldsBase(__base__):
-        def __init__(self, *args, **kwargs):
-            arguments = OrderedDict(zip(everything, args))
-            if len(args) > len(everything):
-                raise TypeError("__init__() takes at most %s arguments (%s given)" % (len(everything), len(args)))
-            for name, value in kwargs.items():
-                if name in arguments:
-                    raise TypeError("__init__() got multiple values for keyword argument %r" % name)
-                elif name not in everything:
-                    raise TypeError("__init__() got an unexpected keyword argument %r" % name)
-                else:
-                    arguments[name] = value
-            for pos, name in enumerate(required):
-                if name not in arguments:
-                    raise TypeError("__init__() requires argument %r (at position %s)" % (name, pos))
-            if defaults:
-                for name, value in defaults.items():
-                    if name not in arguments:
-                        arguments[name] = value
-            for name, value in arguments.items():
-                setattr(self, name, value)
+    global_namespace, local_namespace = _make_init_func(required, defaults, everything)
 
-            super(FieldsBase, self).__init__(*args, **kwargs)
+    class FieldsBase(__base__):
+        __init__ = local_namespace['__init__']
 
         def __eq__(self, other):
             if isinstance(other, self.__class__):
@@ -101,6 +108,7 @@ def class_sealer(required, defaults, everything):
                 self.__class__.__name__,
                 ", ".join(a + "=" + repr(getattr(self, a)) for a in everything)
             )
+    global_namespace['FieldsBase'] = FieldsBase
     return FieldsBase
 
 
@@ -252,45 +260,21 @@ class Factory(type):
         return cls.__concrete
 
 
-class ValidationError(Exception):
-    pass
-
-
-def regex_validation_sealer(required, defaults, everything, RegexType=type(re.compile(""))):
+class Namespace(object):
     """
-    Example sealer that just does regex-based validation.
+    A backport of Python 3.3's ``types.SimpleNamespace``.
     """
-    if required:
-        raise TypeError("regex_validation_sealer doesn't support required arguments")
+    def __init__(self, **kwargs):
+        self.__dict__.update(kwargs)
 
-    klass = None
-    kwarg_validators = dict(
-        (key, val if isinstance(val, RegexType) else re.compile(val)) for key, val in defaults.items()
-    )
-    arg_validators = list(
-        kwarg_validators[key] for key in everything
-    )
+    def __repr__(self):
+        keys = sorted(self.__dict__)
+        items = ("{}={!r}".format(k, self.__dict__[k]) for k in keys)
+        return "{}({})".format(type(self).__name__, ", ".join(items))
 
-    def __init__(self, *args, **kwargs):
-        for pos, (value, validator) in enumerate(zip(args, arg_validators)):
-            if not validator.match(value):
-                raise ValidationError("Positional argument %s failed validation. %r doesn't match regex %r" % (
-                    pos, value, validator.pattern
-                ))
-        for key, value in kwargs.items():
-            if key in kwarg_validators:
-                validator = kwarg_validators[key]
-                if not validator.match(value):
-                    raise ValidationError("Keyword argument %r failed validation. %r doesn't match regex %r" % (
-                        key, value, validator.pattern
-                    ))
-        super(klass, self).__init__(*args, **kwargs)
+    def __eq__(self, other):
+        return self.__dict__ == other.__dict__
 
-    klass = type("RegexValidateBase", (__base__,), dict(
-        __init__=__init__,
-    ))
-    return klass
 
 Fields = Factory()
 Tuple = Factory(sealer=Callable(tuple_sealer))
-RegexValidate = Factory(sealer=Callable(regex_validation_sealer))
